"""The future strategy implements getting data from LibreTime."""

from datetime import datetime


def get_show(future: datetime) -> str:  # pragma: no cover
    """Return the currently runing shows slug from LibreTime."""
    raise NotImplementedError("future")



"""Main "get" interface for RaBe CRID's."""

from datetime import datetime

from .lib import CRID
from .strategy import future, now, past


def get(timestamp=None) -> CRID:
    """Get a RaBe CRID."""
    _now = datetime.now()
    _ts = timestamp or _now
    _show = ""
    if _ts == _now:
        _show = now.get_show()
    elif _ts < _now:
        _show = past.get_show(past=_ts)
    elif _ts > _now:  # pragma: no cover
        _show = future.get_show(future=_ts)

    _tscode = f"{_ts.strftime('%Y%m%dT%H%M%S.%f')[:-4]}Z"
    return CRID(f"crid://rabe.ch/v1/{_show}#t=clock={_tscode}")



"""Low-level cridlib implemenetation bits."""

from datetime import datetime
from pathlib import PurePath
from urllib.parse import parse_qs

from uritools import uricompose, urisplit  # type: ignore


class CRIDError(Exception):
    """Represent all cridlib errors."""


class CRIDSchemeMismatchError(CRIDError):
    """Scheme in URI does not match 'crid'."""


class CRIDSchemeAuthorityMismatchError(CRIDError):
    """Hostname part of URI does not match 'rabe.ch'."""


class CRIDUnsupportedVersionError(CRIDError):
    """Unsupported version in path of URI."""


class CRIDMissingMediaFragmentError(CRIDError):
    """Missing media-fragment with clock code."""


class CRIDMalformedMediaFragmentError(CRIDError):
    """Missing media-fragment with clock code."""


class CRID:
    """Represent CRIDs using uri."""

    def __init__(self, _uri=None) -> None:
        self._uri = urisplit(_uri)
        if self.scheme != "crid":
            raise CRIDSchemeMismatchError(self.scheme)
        if self.authority != "rabe.ch":
            raise CRIDSchemeAuthorityMismatchError(self.authority)
        if self.path.parent.stem != "v1":
            raise CRIDUnsupportedVersionError(self.path)
        self._version = self.path.parent.stem
        self._show = self.path.relative_to(self.path.parent).stem
        try:
            self._start = datetime.strptime(
                parse_qs(parse_qs(self.fragment)["t"][0])["clock"][0],
                "%Y%m%dT%H%M%S.%fZ",
            )
        except KeyError as ex:
            raise CRIDMissingMediaFragmentError(self.fragment) from ex
        except ValueError as ex:  # pragma: no cover
            raise CRIDMalformedMediaFragmentError(self.fragment) from ex

    def __str__(self):
        return uricompose(*self._uri)

    def __repr__(self) -> str:
        """Pretty print CRID.

        >>> CRID("crid://rabe.ch/v1/test#t=clock=19930301T131200.00Z")
        <class 'cridlib.lib.CRID' for 'crid://rabe.ch/v1/test#t=clock=19930301T131200.00Z'>

        """
        _fqcn = f"{self.__class__.__module__}.{self.__class__.__qualname__}"
        return f"<class '{_fqcn}' for '{str(self)}'>"

    @property
    def scheme(self):
        """Get RaBe CRID scheme."""
        return self._uri.scheme

    @property
    def authority(self):
        """Get RaBe CRID authority."""
        return self._uri.authority

    @property
    def path(self):
        """Get RaBe CRID path."""
        return PurePath(self._uri.path)

    @property
    def fragment(self):
        """Get RaBe CRID fragment."""
        return self._uri.fragment

    @property
    def version(self):
        """Get RaBe CRID version."""
        return self._version

    @property
    def show(self):
        """Get RaBe CRID show."""
        return self._show

    @property
    def start(self):
        """Get RaBe CRID start time."""
        return self._start



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cridlib', 'cridlib.strategy']

package_data = \
{'': ['*']}

install_requires = \
['pytz>=2022.6', 'requests>=2.28.1,<3.0.0', 'uritools>=4.0.0,<5.0.0']

setup_kwargs = {
    'name': 'rabe-cridlib',
    'version': '0.1.7',
    'description': 'Generate CRIDs for RaBe',
    'long_description': '# RaBe cridlib for Python\n\nGenerate [RaBe CRIDs](https://github.com/radiorabe/crid-spec) based on several data sources:\n\n* Songticker for current CRID\n* `archiv.rabe.ch` for past CRIDs\n* TBD: LibreTime for future CRIDs\n\n## Installation\n\n```bash\npoetry add rabe-cridlib\n\n# or on old setup style projects\npip -m install rabe-cridlib\n```\n\n## Usage\n\n```python\n>>> import cridlib\n>>>\n>>> # parse an existing crid\n>>> crid = cridlib.parse("crid://rabe.ch/v1/klangbecken#t=clock=19930301T131200.00Z")\n>>> print(f"version: {crid.version}, show: {crid.show}, start: {crid.start}")\nversion: v1, show: klangbecken, start: 1993-03-01 13:12:00\n\n>>> # get crid for current show\n>>> crid = cridlib.get()\n>>> print(f"version: {crid.version}, show: {crid.show}")  # doctest:+ELLIPSIS\nversion: v1, show: ...\n\n```\n\n## Development\n\n```bash\n# setup a dev env\npython -mvenv env\n. env/bin/activate\n\n# install a modern poetry version\npython -mpip install poetry>=1.2.0\n\n# install deps and dev version\npoetry install\n\n# make changes, run tests\npytest\n```\n\n## Release Management\n\nThe CI/CD setup uses semantic commit messages following the [conventional commits standard](https://www.conventionalcommits.org/en/v1.0.0/).\nThere is a GitHub Action in [.github/workflows/semantic-release.yaml](./.github/workflows/semantic-release.yaml)\nthat uses [go-semantic-commit](https://go-semantic-release.xyz/) to create new\nreleases.\n\nThe commit message should be structured as follows:\n\n```\n<type>[optional scope]: <description>\n\n[optional body]\n\n[optional footer(s)]\n```\n\nThe commit contains the following structural elements, to communicate intent to the consumers of your library:\n\n1. **fix:** a commit of the type `fix` patches gets released with a PATCH version bump\n1. **feat:** a commit of the type `feat` gets released as a MINOR version bump\n1. **BREAKING CHANGE:** a commit that has a footer `BREAKING CHANGE:` gets released as a MAJOR version bump\n1. types other than `fix:` and `feat:` are allowed and don\'t trigger a release\n\nIf a commit does not contain a conventional commit style message you can fix\nit during the squash and merge operation on the PR.\n\nOnce a commit has landed on the `main` branch a release will be created and automatically published to [pypi](https://pypi.org/)\nusing the GitHub Action in [.github/workflows/release.yaml](./.github/workflows/release.yaml) which uses [poetry](https://python-poetry.org/)\nto publish the package to pypi.\n\n## License\n\nThis package is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, version 3 of the License.\n\n## Copyright\n\nCopyright (c) 2022 [Radio Bern RaBe](http://www.rabe.ch)\n',
    'author': 'RaBe IT-Reaktion',
    'author_email': 'it@rabe.ch',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/radiorabe/python-rabe-cridlib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""The now strategy implements getting data from Songticker."""

import xml.etree.ElementTree as ET
from pathlib import PurePath

import requests
from uritools import urisplit  # type: ignore

from cridlib.const import SONGTICKER_URL


def get_show() -> str:
    """Return the currently running shows slug from nowplaying."""

    _resp = requests.get(SONGTICKER_URL, timeout=10)
    _tree = ET.fromstring(_resp.text)
    _path = PurePath(urisplit(_tree[3][1].text).path)
    return _path.stem



"""The past strategy implements getting data from archiv.rabe.ch."""

from datetime import datetime

import requests

from cridlib.const import ARCHIV_BROADCASTS_URL


def get_show(past: datetime) -> str:
    """Return the a show slug from a past show."""

    _url = f"{ARCHIV_BROADCASTS_URL}{past.year}/{past.month:02d}/{past.day:02d}/{past.hour:02d}{past.minute:02d}{past.second:02d}"  # pylint: disable=line-too-long
    _resp = requests.get(_url, timeout=10)
    _json = _resp.json()
    _data = _json.get("data")
    _label = str(_data[0].get("attributes").get("label"))

    return _label.lower().replace(" ", "-")
